@@ -1,14 +1,14 @@
 ;;; zk-index.el --- Index for zk   -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2022-2024  Grant Rosson
+;; Copyright (C) 2022-2025  Grant Rosson
 
 ;; Author: Grant Rosson <https://github.com/localauthor>
 ;; Created: January 25, 2022
 ;; License: GPL-3.0-or-later
-;; Package-Version: 0.7
-;; Package-Revision: 599be69ae1c1
-;; Homepage: https://github.com/localauthor/zk
-;; Package-Requires: ((emacs "27.1")(zk "0.7"))
+;; Package-Version: 20250908.1240
+;; Package-Revision: 302e49432406
+;; URL: https://github.com/localauthor/zk
+;; Package-Requires: ((emacs "28.1")(zk "0.9"))
 
 ;; This program is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by the Free
@@ -46,6 +46,7 @@
 
 (defcustom zk-index-buffer-name "*ZK-Index*"
   "Name for ZK-Index buffer."
+  :local t
   :type 'string)
 
 (defcustom zk-index-format-function 'zk-index--format-candidates
@@ -73,6 +74,11 @@ Set to nil to inhibit help-echo."
   "Enable automatically showing note at point in ZK-Index."
   :type 'boolean)
 
+(defcustom zk-index-view-debounce-delay 0.40
+  "Seconds to wait before auto viewing note at point in ZK-Index.
+Only relevant when `zk-index-auto-scroll’ is non-nil."
+  :type 'number)
+
 (defcustom zk-index-cursor nil
   "Cursor to use when `zk-index’ is in the selected window.
 See `cursor-type’ for description of possible values."
@@ -119,6 +125,9 @@ example."
     (define-key map (kbd "S") #'zk-index-sort-size)
     (define-key map (kbd "M") #'zk-index-sort-modified)
     (define-key map (kbd "C") #'zk-index-sort-created)
+    (define-key map (kbd "T") #'zk-index-modified-today)
+    (define-key map (kbd "Y") #'zk-index-modified-yesterday)
+    (define-key map (kbd "W") #'zk-index-modified-this-week)
     (define-key map (kbd "RET") #'zk-index-open-note)
     (define-key map (kbd "q") #'delete-window)
     (make-composed-keymap map tabulated-list-mode-map))
@@ -129,6 +138,7 @@ example."
 \\{zk-index-mode-map}"
   (read-only-mode)
   (hl-line-mode)
+  (setq-local zk-index-buffer-name (buffer-name))
   (setq-local show-paren-mode nil)
   (setq-local cursor-type zk-index-cursor))
 
@@ -137,8 +147,7 @@ example."
 
 (defvar zk-index-last-sort-function nil)
 (defvar zk-index-last-format-function nil)
-(defvar zk-index-query-mode-line nil)
-(defvar zk-index-query-terms nil)
+(defvar-local zk-index-query-mode-line nil)
 (defvar zk-search-history)
 (defvar zk--no-gc)
 
@@ -196,8 +205,8 @@ all files in `zk-directory' will be returned as formatted candidates."
   (let* ((format (or format zk-index-format))
          (list (or files
                    (zk--directory-files)))
-         (output))
-    (dolist (file list)
+         output)
+    (dolist (file list output)
       (when (string-match (zk-file-name-regexp) file)
         (let ((id (if zk-index-invisible-ids
                       (propertize (match-string 1 file) 'invisible t)
@@ -206,8 +215,7 @@ all files in `zk-directory' will be returned as formatted candidates."
                       zk-file-name-separator
                       " "
                       (match-string 2 file))))
-          (push (zk--format format id title) output))))
-    output))
+          (push (zk--format format id title) output))))))
 
 ;;; Main Stack
 
@@ -216,13 +224,15 @@ all files in `zk-directory' will be returned as formatted candidates."
   "Open ZK-Index, with optional FILES, FORMAT-FN, SORT-FN, BUF-NAME."
   (interactive)
   (setq zk-index-last-format-function format-fn)
-  (setq zk-index-last-sort-function sort-fn)
-  (let ((inhibit-message nil)
-        (inhibit-read-only t)
-        (buf-name (or buf-name
-                      zk-index-buffer-name))
-        (list (or files
-                  (zk--directory-files t))))
+  (when sort-fn
+    (setq zk-index-last-sort-function sort-fn))
+  (let* ((zk--no-gc t)
+         (inhibit-message nil)
+         (inhibit-read-only t)
+         (buf-name (or buf-name
+                       zk-index-buffer-name))
+         (files (or files
+                    (zk--directory-files t))))
     (if (not (get-buffer buf-name))
         (progn
           (when zk-default-backlink
@@ -232,7 +242,7 @@ all files in `zk-directory' will be returned as formatted candidates."
           (with-current-buffer buf-name
             (setq default-directory (expand-file-name zk-directory))
             (zk-index-mode)
-            (zk-index--sort list format-fn sort-fn)
+            (zk-index--sort files format-fn sort-fn)
             (setq truncate-lines t)
             (goto-char (point-min)))
           (pop-to-buffer buf-name
@@ -255,7 +265,7 @@ Optionally refresh with FILES, using FORMAT-FN, SORT-FN, BUF-NAME."
                       (setq zk-index-last-sort-function nil)))
          (buf-name (or buf-name
                        zk-index-buffer-name))
-         (pos))
+         pos)
     (setq zk-index-last-format-function format-fn)
     (setq zk-index-last-sort-function sort-fn)
     (with-current-buffer buf-name
@@ -266,9 +276,8 @@ Optionally refresh with FILES, using FORMAT-FN, SORT-FN, BUF-NAME."
       (goto-char (point-min))
       (setq truncate-lines t)
       (unless (zk-index-narrowed-p buf-name)
-        (progn
-          (zk-index--reset-mode-line)
-          (goto-char pos))))))
+        (zk-index--reset-mode-line)
+        (goto-char pos)))))
 
 (defun zk-index--sort (files &optional format-fn sort-fn)
   "Sort FILES, with option FORMAT-FN and SORT-FN."
@@ -286,27 +295,35 @@ Optionally refresh with FILES, using FORMAT-FN, SORT-FN, BUF-NAME."
          (candidates (funcall format-fn files)))
     (zk-index--insert candidates)))
 
+(eval-and-compile
+  (defvar zk-index-button-map
+    (let ((map (make-sparse-keymap)))
+      (set-keymap-parent map button-map)
+      map)
+    "Keymap for ZK-Index buttons."))
+
 (eval-and-compile
   (define-button-type 'zk-index
     'follow-link t
     'action 'zk-index-button-action
+    'keymap 'zk-index-button-map
     'face 'default))
 
 (defun zk-index--insert (candidates)
   "Insert CANDIDATES into ZK-Index."
-  (when (eq major-mode 'zk-index-mode)
+  (when (derived-mode-p 'zk-index-mode)
     (save-excursion
       (dolist (file candidates)
         (insert zk-index-prefix file "\n"))
       (zk-index--make-buttons)
-      (zk-index--set-mode-name (format "[%s]" (length candidates))))))
+      (zk-index--set-mode-name (format " [%s]" (length candidates))))))
 
 (defun zk-index--make-buttons ()
   "Make buttons in ZK-Index buffer."
   (goto-char (point-min))
   (while (re-search-forward zk-id-regexp nil t)
-    (let* ((beg (line-beginning-position))
-           (end (line-end-position)))
+    (let ((beg (line-beginning-position))
+          (end (line-end-position)))
       (beginning-of-line)
       (make-text-button beg end
                         'type 'zk-index
@@ -344,82 +361,85 @@ Optionally refresh with FILES, using FORMAT-FN, SORT-FN, BUF-NAME."
 
 (defun zk-index-narrowed-p (buf-name)
   "Return t when index is narrowed in buffer BUF-NAME."
-  (with-current-buffer (or buf-name
-                           zk-index-buffer-name)
-    (if (< (count-lines (point-min) (point-max))
-           (length (zk--directory-files)))
-        t nil)))
+  (when (get-buffer buf-name)
+    (with-current-buffer buf-name
+      (if (< (count-lines (point-min) (point-max))
+             (length (zk--directory-files)))
+          t nil))))
 
 ;;; Index Search and Focus Functions
 
 ;;;; Index Search
 ;; narrow index based on search of notes' full text
 
-(defun zk-index-search ()
-  "Narrow index based on regexp search of note contents."
-  (interactive)
-  (if (eq major-mode 'zk-index-mode)
-      (zk-index-refresh
-       (zk-index-query-files)
-       zk-index-last-format-function
-       zk-index-last-sort-function
-       (buffer-name))
-    (user-error "Not in a ZK-Index")))
+(defun zk-index-search (string)
+  "Narrow index to notes containing STRING."
+  (interactive (list
+                (read-string "Narrow index by content: "
+                             nil 'zk-search-history)))
+  (when (zk-index-query-files string)
+    (zk-index-switch-to-index)))
 
 ;;;; Index Focus
 ;; narrow index based on search of note titles (case sensitive)
 ;; an alternative to consult-focus-lines
 
-(defun zk-index-focus ()
-  "Narrow index based on regexp search of note titles."
-  (interactive)
-  (if (eq major-mode 'zk-index-mode)
-      (zk-index-refresh
-       (zk-index-query-files)
-       zk-index-last-format-function
-       zk-index-last-sort-function
-       (buffer-name))
-    (user-error "Not in a ZK-Index")))
+(defun zk-index-focus (string)
+  "Narrow index to notes with STRING in title."
+  (interactive (list
+                (read-string "Narrow index by title: "
+                             nil 'zk-search-history)))
+  (when (zk-index-query-files string)
+    (zk-index-switch-to-index)))
 
 ;;;; Low-level Query Functions
 
-(defvar zk-index-query-terms nil
+(defvar-local zk-index-query-terms nil
   "Ordered list of current query terms.
 Takes form of (COMMAND . TERM), where COMMAND is `ZK-INDEX-FOCUS
 or `ZK-INDEX-SEARCH, and TERM is the query string. Recent
 items listed first.")
 
-(defun zk-index-query-files ()
-  "Return narrowed list of notes, based on focus or search query."
+(defun zk-index-query-files (string)
+  "Return narrowed list of notes, based on focus or search query for STRING."
   (let* ((zk--no-gc t)
-         (command this-command)
-         (scope (if (zk-index-narrowed-p (buffer-name))
-                    (zk-index--current-id-list (buffer-name))
-                  (setq zk-index-query-terms nil)
+         (command (if (eq this-command 'zk-index-focus)
+                      'zk-index-focus
+                    'zk-index-search))
+         (index-buf zk-index-buffer-name)
+         (scope (if (zk-index-narrowed-p index-buf)
+                    (zk-index--current-id-list index-buf)
+                  (setq-local zk-index-query-terms nil)
                   (zk--id-list)))
-         (string (read-string (cond ((eq command 'zk-index-focus)
-                                     "Focus: ")
-                                    ((eq command 'zk-index-search)
-                                     "Search: "))
-                              nil 'zk-search-history))
+         (string (or string
+                     (read-string "Narrow by title: "
+                                  nil 'zk-search-history)))
          (query (cond
+                 ((string-empty-p string)
+                  (user-error "Must enter a search string"))
                  ((eq command 'zk-index-focus)
                   (zk--id-list string))
-                 ((eq command 'zk-index-search)
+                 (t
                   (zk--grep-id-list string))))
+         (zk-mode-line)
          (ids (mapcar (lambda (x) (when (member x scope) x))
                       query))
-         (files (zk--parse-id 'file-path (remq nil ids))))
+         (files
+          (ensure-list (zk--parse-id 'file-path (remq nil ids)))))
     (add-to-history 'zk-search-history string)
-    (when files
-      (setq zk-index-query-mode-line
-            (zk-index-query-mode-line command string))
-      (add-to-list 'mode-line-misc-info '(:eval (zk-index--query-mode-line)))
-      (force-mode-line-update t))
-    (when (stringp files)
-      (setq files (list files)))
-    (or files
-        (error "No matches for \"%s\"" string))))
+    (if files
+        (progn
+          (setq zk-mode-line (zk-index-query-mode-line command string))
+          (zk-index files
+                    zk-index-last-format-function
+                    zk-index-last-sort-function
+                    index-buf)
+          (setq zk-index-query-mode-line
+                (propertize zk-mode-line
+                            'help-echo zk-mode-line))
+          (add-to-list 'mode-line-misc-info '(:eval (zk-index--query-mode-line)))
+          (force-mode-line-update t))
+      (error "No matches for \"%s\" in %s" string zk-index-buffer-name))))
 
 (defun zk-index-query-refresh ()
   "Refresh narrowed index, based on last focus or search query."
@@ -457,9 +477,11 @@ with query term STRING."
               (mapconcat (lambda (query)
                            (when query
                              (concat
-                              (capitalize
-                               (caddr
-                                (split-string (symbol-name (car query)) "-")))
+                              (cond
+                               ((eq (car query) 'zk-index-focus)
+                                "Title")
+                               ((eq (car query) 'zk-index-search)
+                                "Content"))
                               ": \""
                               (cdr query))))
                          ;; Put the last query type at the end
@@ -471,13 +493,13 @@ with query term STRING."
 
 (defun zk-index--query-mode-line ()
   "Add STRING to mode-line in `zk-index-mode'."
-  (when (eq major-mode 'zk-index-mode)
+  (when (derived-mode-p 'zk-index-mode)
     zk-index-query-mode-line))
 
 (defun zk-index--reset-mode-line ()
   "Reset mode-line in `zk-index-mode'."
-  (setq zk-index-query-mode-line nil
-        zk-index-query-terms nil))
+  (setq-local zk-index-query-mode-line nil
+              zk-index-query-terms nil))
 
 (defun zk-index--current-id-list (buf-name)
   "Return list of IDs for index in BUF-NAME, as filepaths."
@@ -488,14 +510,14 @@ with query term STRING."
         (goto-char (point-min))
         (save-match-data
           (while (re-search-forward zk-id-regexp nil t)
-            (push (match-string-no-properties 0) ids)))
+            (push (match-string-no-properties 1) ids)))
         ids))))
 
 ;;; Index Sort Functions
 
 (defun zk-index--sort-call (sort-fn mode-string)
   "Call SORT-FN with MODE-STRING."
-  (if (eq major-mode 'zk-index-mode)
+  (if (derived-mode-p 'zk-index-mode)
       (let ((zk--no-gc t))
         (zk-index-refresh (zk-index--current-file-list)
                           zk-index-last-format-function
@@ -507,22 +529,21 @@ with query term STRING."
 (defun zk-index-sort-modified ()
   "Sort index by last modified."
   (interactive)
-  (zk-index--sort-call #'zk-index--sort-modified "by modified"))
+  (zk-index--sort-call #'zk-index--sort-modified " by modified"))
 
 (defun zk-index-sort-created ()
   "Sort index by date created."
   (interactive)
-  (zk-index--sort-call #'zk-index--sort-created "by created"))
+  (zk-index--sort-call #'zk-index--sort-created " by created"))
 
 (defun zk-index-sort-size ()
   "Sort index by size."
   (interactive)
-  (zk-index--sort-call #'zk-index--sort-size "by size"))
+  (zk-index--sort-call #'zk-index--sort-size " by size"))
 
 (defun zk-index--set-mode-name (string)
   "Add STRING to `mode-name' in `zk-index-mode'."
-  (when (eq major-mode 'zk-index-mode)
-    (setq mode-name (concat mode-name " " string))))
+  (setq mode-name (concat mode-name string)))
 
 (defun zk-index--reset-mode-name ()
   "Reset `mode-name' in `zk-index-mode'."
@@ -530,44 +551,72 @@ with query term STRING."
 
 (defun zk-index--current-file-list ()
   "Return list files in current index."
-  (let* ((ids (zk-index--current-id-list (buffer-name)))
-         (files (zk--parse-id 'file-path ids)))
-    (when files
-      files)))
+  (let ((ids (zk-index--current-id-list (buffer-name))))
+    (zk--parse-id 'file-path ids)))
 
-(defun zk-index--sort-created (list)
-  "Sort LIST for latest created."
-  (let ((ht (make-hash-table :test #'equal :size 5000)))
-    (dolist (x list)
+(defun zk-index--sort-created (files)
+  "Sort list of FILES by latest created."
+  (let ((ht (make-hash-table :test #'equal :size (length files))))
+    (dolist (x files)
       (puthash x (zk--parse-file 'id x) ht))
-    (sort list
+    (sort files
           (lambda (a b)
             (string>
              (gethash a ht)
              (gethash b ht))))))
 
-(defun zk-index--sort-modified (list)
-  "Sort LIST for latest modification."
-  (let ((ht (make-hash-table :test #'equal :size 5000)))
-    (dolist (x list)
+(defun zk-index--sort-modified (files)
+  "Sort list of FILES by latest modification."
+  (let ((ht (make-hash-table :test #'equal :size (length files))))
+    (dolist (x files)
       (puthash x (file-attribute-modification-time (file-attributes x)) ht))
-    (sort list
+    (sort files
           (lambda (a b)
             (time-less-p
              (gethash b ht)
              (gethash a ht))))))
 
-(defun zk-index--sort-size (list)
-  "Sort LIST for latest modification."
-  (let ((ht (make-hash-table :test #'equal :size 5000)))
-    (dolist (x list)
+(defun zk-index--sort-size (files)
+  "Sort list of FILES by latest modification."
+  (let ((ht (make-hash-table :test #'equal :size (length files))))
+    (dolist (x files)
       (puthash x (file-attribute-size (file-attributes x)) ht))
-    (sort list
+    (sort files
           (lambda (a b)
             (>
              (gethash a ht)
              (gethash b ht))))))
 
+;;; Modified on Date Functions
+
+(defun zk-index-modified-today ()
+  "Open ZK-Index with notes modified today."
+  (interactive)
+  (if-let ((files (zk--files-modified-today)))
+      (progn
+        (zk-index files)
+        (zk-index--set-mode-name " today"))
+    (user-error "No files modified today")))
+
+(defun zk-index-modified-yesterday ()
+  "Open ZK-Index with notes modified yesterday."
+  (interactive)
+  (if-let ((files (zk--files-modified-yesterday)))
+      (progn
+        (zk-index files)
+        (zk-index--set-mode-name " yesterday"))
+    (user-error "No files modified yesterday")))
+
+(defun zk-index-modified-this-week ()
+  "Open ZK-Index with notes modified this week."
+  (interactive)
+  (if-let ((files (zk--files-modified-this-week)))
+      (progn
+        (zk-index files)
+        (zk-index--set-mode-name " this week"))
+    (user-error "No files modified this week")))
+
+
 ;;; ZK-Index Keymap Commands
 
 (defun zk-index-open-note ()
@@ -582,10 +631,12 @@ with query term STRING."
   "View note in `zk-index-view-mode'."
   (interactive)
   (beginning-of-line)
-  (let* ((zk-enable-link-buttons nil)
+  (let* ((zk--no-gc t)
+         (zk-enable-link-buttons nil)
          (id (zk-index--button-at-point-p))
          (file (zk--parse-id 'file-path id))
-         (kill (unless (get-file-buffer file)
+         (kill (unless (and file
+                            (get-file-buffer file))
                  t))
          (buffer (find-file-noselect file))
          (index-name (buffer-name)))
@@ -605,13 +656,15 @@ with query term STRING."
 (defun zk-index--button-at-point-p (&optional pos)
   "Return zk-id when `zk-index' button is at point.
 Takes an option POS position argument."
-  (let ((button (or pos
-                    (button-at (point)))))
-    (when (and button
-               (button-has-type-p button 'zk-index))
-      (save-excursion
-        (re-search-forward zk-id-regexp)
-        (match-string-no-properties 1)))))
+  (save-excursion
+    (beginning-of-line)
+    (let ((button (or pos
+                      (button-at (point)))))
+      (when (and button
+                 (button-has-type-p button 'zk-index))
+        (save-excursion
+          (when (re-search-forward zk-id-regexp)
+            (match-string-no-properties 1)))))))
 
 (defun zk-index-insert-link (&optional id)
   "Insert zk-link in `other-window' for button ID at point."
@@ -636,17 +689,22 @@ Takes an option POS position argument."
             ((kbd "p") . zk-index-previous-line)
             ([return] . zk-index-view-mode)
             ([remap read-only-mode] . zk-index-view-mode)
-            ((kbd "q") . quit-window))
+            ((kbd "s") . zk-index-search)
+            ((kbd "/") . zk-index-focus)
+            ((kbd "q") . zk-index-view-quit-window))
   (if zk-index-view-mode
-      (progn
-        (read-only-mode)
-        (when zk-index-view-hide-cursor
+      (if (zk-file-p)
           (progn
-            (scroll-lock-mode 1)
-            (setq-local zk-index-view--cursor
-                        cursor-type)
-            (setq-local cursor-type nil))))
+            (read-only-mode)
+            (when zk-index-view-hide-cursor
+              (progn
+                (scroll-lock-mode 1)
+                (setq-local zk-index-view--cursor
+                            cursor-type)
+                (setq-local cursor-type nil))))
+        (error "Not a zk file"))
     (read-only-mode -1)
+    (kill-local-variable 'zk-index-buffer-name)
     (when zk-enable-link-buttons
       (zk-make-link-buttons))
     (when zk-index-view-hide-cursor
@@ -654,26 +712,50 @@ Takes an option POS position argument."
       (setq-local cursor-type (or zk-index-view--cursor
                                   t)))))
 
+(defun zk-index-view-quit-window ()
+  "Quit `zk-index-view-mode’ window and select `zk-index’ buffer."
+  (interactive)
+  (let ((index zk-index-buffer-name))
+    ;; mode-off kills local value of zk-index-buffer-name,
+    ;; so need to let bind it, to avoid switching to default index
+    (zk-index-view-mode -1)
+    (quit-window zk-index-view--kill)
+    (zk-index-switch-to-index index)))
+
+(defvar zk-index--debounce-timer nil)
+
+(defun zk-index--view-note-debounce ()
+  "Delay calling of `zk-index-view-note’ by."
+  (when (timerp zk-index--debounce-timer)
+    (cancel-timer zk-index--debounce-timer))
+  (setq zk-index--debounce-timer
+        (run-with-idle-timer
+         zk-index-view-debounce-delay nil
+         (lambda ()
+           (setq zk-index--debounce-timer nil)
+           (other-window 1)
+           (when (zk-file-p)
+             (if zk-index-view--kill
+                 (kill-buffer)
+               (zk-index-view-mode -1)))
+           (other-window 1)
+           (zk-index-view-note)))))
+
 (defun zk-index-forward-button (N)
   "Move to the Nth next button, or Nth previous button if N is negative.
 If `zk-index-auto-scroll' is non-nil, show note in other window."
   (let ((split-width-threshold nil)
-        (index-name zk-index-buffer-name))
+        (index-window (get-buffer-window
+                       zk-index-buffer-name)))
     (if zk-index-auto-scroll
         (progn
-          (cond ((not (zk-file-p)))
-                (zk-index-view--kill
-                 (kill-buffer)
-                 (select-window (get-buffer-window
-                                 index-name)))
-                ((not zk-index-view--kill)
-                 (zk-index-view-mode)
-                 (select-window (get-buffer-window
-                                 index-name))))
+          (if (and (zk-file-p)
+                   index-window)
+              (select-window index-window))
           (forward-button N)
           (hl-line-highlight)
           (unless (looking-at-p "[[:space:]]*$")
-            (zk-index-view-note)))
+            (zk-index--view-note-debounce)))
       (forward-button N))))
 
 (defun zk-index-next-line ()
@@ -689,15 +771,22 @@ If `zk-index-auto-scroll' is non-nil, show note in other window."
   (zk-index-forward-button -1))
 
 ;;;###autoload
-(defun zk-index-switch-to-index ()
-  "Switch to ZK-Index buffer."
+(defun zk-index-switch-to-index (&optional index)
+  "Switch to ZK-Index buffer INDEX.
+Will switch to local value of `zk-index-buffer-name’."
   (interactive)
-  (let ((buffer zk-index-buffer-name))
-    (unless (get-buffer buffer)
-      (progn
-        (generate-new-buffer buffer)
-        (zk-index-refresh)))
-    (switch-to-buffer buffer)))
+  (if-let ((buffer (get-buffer
+                    (or index
+                        zk-index-buffer-name))))
+      (pop-to-buffer buffer)
+    (zk-index)))
+
+(defun zk-index-rename (buf-name)
+  "Rename current ZK-Index buffer to BUF-NAME."
+  (interactive (list (read-string "Rename index: " (buffer-name))))
+  (when (derived-mode-p 'zk-index-mode)
+    (rename-buffer buf-name)
+    (setq-local zk-index-buffer-name buf-name)))
 
 (provide 'zk-index)
 
